@@ -4,6 +4,7 @@
 from trytond.model import ModelView, ModelSQL, fields
 from trytond.wizard import Wizard
 from trytond.report import Report
+from trytond.tools import reduce_ids
 from decimal import Decimal
 import datetime
 import time
@@ -99,6 +100,7 @@ class TypeTemplate(ModelSQL, ModelView):
         :return: id of the type created
         '''
         type_obj = self.pool.get('account.account.type')
+        lang_obj = self.pool.get('ir.lang')
 
         if template2type is None:
             template2type = {}
@@ -112,6 +114,22 @@ class TypeTemplate(ModelSQL, ModelView):
             vals['parent'] = parent_id
 
             new_id = type_obj.create(cursor, user, vals, context=context)
+
+            prev_lang = template._context.get('language') or 'en_US'
+            ctx = context.copy()
+            for lang in lang_obj.get_translatable_languages(cursor, user,
+                    context=context):
+                if lang == prev_lang:
+                    continue
+                ctx['language'] = lang
+                template.setLang(lang)
+                data = {}
+                for field in template._columns.keys():
+                    if template._columns[field].translate:
+                        data[field] = template[field]
+                if data:
+                    type_obj.write(cursor, user, new_id, data, context=ctx)
+            template.setLang(prev_lang)
             template2type[template.id] = new_id
         else:
             new_id = template2type[template.id]
@@ -240,6 +258,7 @@ class Type(ModelSQL, ModelView):
                 into type. The dictionary is filled with new types
         '''
         template_obj = self.pool.get('account.account.type.template')
+        lang_obj = self.pool.get('ir.lang')
 
         if template2type is None:
             template2type = {}
@@ -252,6 +271,20 @@ class Type(ModelSQL, ModelView):
                     context=context, type=type)
             if vals:
                 self.write(cursor, user, type.id, vals, context=context)
+
+            prev_lang = type._context.get('language') or 'en_US'
+            ctx = context.copy()
+            for lang in lang_obj.get_translatable_languages(cursor, user,
+                    context=context):
+                if lang == prev_lang:
+                    continue
+                ctx['language'] = lang
+                type.setLang(lang)
+                data = template_obj._get_type_value(cursor, user,
+                        type.template, context=ctx, type=type)
+                if data:
+                    self.write(cursor, user, type.id, data, context=ctx)
+            type.setLang(prev_lang)
             template2type[type.template.id] = type.id
 
         for child in type.childs:
@@ -389,6 +422,7 @@ class AccountTemplate(ModelSQL, ModelView):
         :return: id of the account created
         '''
         account_obj = self.pool.get('account.account')
+        lang_obj = self.pool.get('ir.lang')
 
         if template2account is None:
             template2account = {}
@@ -406,6 +440,22 @@ class AccountTemplate(ModelSQL, ModelView):
             vals['type'] = template2type.get(template.type.id, False)
 
             new_id = account_obj.create(cursor, user, vals, context=context)
+
+            prev_lang = template._context.get('language') or 'en_US'
+            ctx = context.copy()
+            for lang in lang_obj.get_translatable_languages(cursor, user,
+                    context=context):
+                if lang == prev_lang:
+                    continue
+                ctx['language'] = lang
+                template.setLang(lang)
+                data = {}
+                for field in template._columns.keys():
+                    if template._columns[field].translate:
+                        data[field] = template[field]
+                if data:
+                    account_obj.write(cursor, user, new_id, data, context=ctx)
+            template.setLang(prev_lang)
             template2account[template.id] = new_id
         else:
             new_id = template2account[template.id]
@@ -498,8 +548,8 @@ class Account(ModelSQL, ModelView):
     debit = fields.Function('get_credit_debit', digits="(16, currency_digits)",
             string='Debit', depends=['currency_digits'])
     reconcile = fields.Boolean('Reconcile',
-            help='Check if the account can be used \n' \
-                    'for reconciliation.',
+            help='Allow move lines of this account \n' \
+                    'to be reconciled.',
             states={
                 'invisible': "kind == 'view'",
             }, depends=['kind'])
@@ -605,6 +655,9 @@ class Account(ModelSQL, ModelView):
                 'GROUP BY a.id', args_ids)
         account_sum = {}
         for account_id, sum in cursor.fetchall():
+            # SQLite uses float for SUM
+            if not isinstance(sum, Decimal):
+                sum = Decimal(str(sum))
             account_sum[account_id] = sum
 
         account2company = {}
@@ -711,22 +764,28 @@ class Account(ModelSQL, ModelView):
 
         line_query, fiscalyear_ids = move_line_obj.query_get(cursor, user,
                 context=context)
-        cursor.execute('SELECT a.id, ' + \
-                    ','.join(['SUM(COALESCE(l.' + name + ', 0))'
-                        for name in names]) + ' ' \
-                'FROM account_account a ' \
-                    'LEFT JOIN account_move_line l ' \
-                    'ON (a.id = l.account) ' \
-                'WHERE a.kind != \'view\' ' \
-                    'AND a.id IN (' + \
-                        ','.join(['%s' for x in ids]) + ') ' \
-                    'AND ' + line_query + ' ' \
-                    'AND a.active ' \
-                'GROUP BY a.id', ids)
-        for row in cursor.fetchall():
-            account_id = row[0]
-            for i in range(len(names)):
-                res[names[i]][account_id] = row[i + 1]
+        for i in range(0, len(ids), cursor.IN_MAX):
+            sub_ids = ids[i:i + cursor.IN_MAX]
+            red_sql, red_ids = reduce_ids('a.id', sub_ids)
+            cursor.execute('SELECT a.id, ' + \
+                        ','.join('SUM(COALESCE(l.' + name + ', 0))'
+                            for name in names) + ' ' \
+                    'FROM account_account a ' \
+                        'LEFT JOIN account_move_line l ' \
+                        'ON (a.id = l.account) ' \
+                    'WHERE a.kind != \'view\' ' \
+                        'AND ' + red_sql + ' ' \
+                        'AND ' + line_query + ' ' \
+                        'AND a.active ' \
+                    'GROUP BY a.id', red_ids)
+            for row in cursor.fetchall():
+                account_id = row[0]
+                for i in range(len(names)):
+                    # SQLite uses float for SUM
+                    if not isinstance(row[i + 1], Decimal):
+                        res[names[i]][account_id] = Decimal(str(row[i + 1]))
+                    else:
+                        res[names[i]][account_id] = row[i + 1]
 
         account2company = {}
         id2company = {}
@@ -817,6 +876,10 @@ class Account(ModelSQL, ModelView):
         return args2
 
     def copy(self, cursor, user, ids, default=None, context=None):
+        if default is None:
+            default = {}
+        default['left'] = 0
+        default['right'] = 0
         res = super(Account, self).copy(cursor, user, ids, default=default,
                 context=context)
         self._rebuild_tree(cursor, user, 'parent', False, 0)
@@ -868,6 +931,7 @@ class Account(ModelSQL, ModelView):
                 into type.
         '''
         template_obj = self.pool.get('account.account.template')
+        lang_obj = self.pool.get('ir.lang')
 
         if template2account is None:
             template2account = {}
@@ -887,6 +951,20 @@ class Account(ModelSQL, ModelView):
                         False)
             if vals:
                 self.write(cursor, user, account.id, vals, context=context)
+
+            prev_lang = account._context.get('language') or 'en_US'
+            ctx = context.copy()
+            for lang in lang_obj.get_translatable_languages(cursor, user,
+                    context=context):
+                if lang == prev_lang:
+                    continue
+                ctx['language'] = lang
+                account.setLang(lang)
+                data = template_obj._get_account_value(cursor, user,
+                           account.template, context=ctx, account=account)
+                if data:
+                    self.write(cursor, user, account.id, data, context=ctx)
+            account.setLang(prev_lang)
             template2account[account.template.id] = account.id
 
         for child in account.childs:
@@ -1069,11 +1147,11 @@ class PrintGeneralLegderInit(ModelView):
     fiscalyear = fields.Many2One('account.fiscalyear', 'Fiscal Year',
             required=True, on_change=['fiscalyear'])
     start_period = fields.Many2One('account.period', 'Start Period',
-            domain="[('fiscalyear', '=', fiscalyear), " \
-                    "('start_date', '<=', (end_period, 'start_date'))]")
+            domain=["('fiscalyear', '=', fiscalyear)",
+                "('start_date', '<=', (end_period, 'start_date'))"])
     end_period = fields.Many2One('account.period', 'End Period',
-            domain="[('fiscalyear', '=', fiscalyear), " \
-                    "('start_date', '>=', (start_period, 'start_date'))]")
+            domain=["('fiscalyear', '=', fiscalyear)",
+                    "('start_date', '>=', (start_period, 'start_date'))"])
     company = fields.Many2One('company.company', 'Company', required=True)
     posted = fields.Boolean('Posted Move', help='Show only posted move')
     empty_account = fields.Boolean('Empty Account',
@@ -1184,16 +1262,12 @@ class GeneralLegder(Report):
                 ('fiscalyear', '=', datas['form']['fiscalyear']),
                 ('end_date', '<=', end_period.start_date),
                 ], context=context)
-            end_period_ids = list(set(end_period_ids).difference(
-                set(start_period_ids)))
             if datas['form']['end_period'] not in end_period_ids:
                 end_period_ids.append(datas['form']['end_period'])
         else:
             end_period_ids = period_obj.search(cursor, user, [
                 ('fiscalyear', '=', datas['form']['fiscalyear']),
                 ], context=context)
-            end_period_ids = list(set(end_period_ids).difference(
-                set(start_period_ids)))
 
         end_context = context.copy()
         end_context['fiscalyear'] = datas['form']['fiscalyear']
@@ -1227,7 +1301,8 @@ class GeneralLegder(Report):
         context['id2end_account'] = id2end_account
         context['digits'] = company.currency.digits
         context['lines'] = lambda account_id: self.lines(cursor, user,
-                account_id, end_period_ids, datas['form']['posted'], context)
+                account_id, list(set(end_period_ids).difference(
+                    set(start_period_ids))), datas['form']['posted'], context)
         context['company'] = company
 
         return super(GeneralLegder, self).parse(cursor, user, report, objects,
@@ -1283,12 +1358,12 @@ class PrintTrialBalanceInit(ModelView):
             required=True, on_change=['fiscalyear'],
             depends=['start_period', 'end_period'])
     start_period = fields.Many2One('account.period', 'Start Period',
-            domain="[('fiscalyear', '=', fiscalyear), " \
-                    "('start_date', '<=', (end_period, 'start_date'))]",
+            domain=["('fiscalyear', '=', fiscalyear)",
+                "('start_date', '<=', (end_period, 'start_date'))"],
             depends=['end_period'])
     end_period = fields.Many2One('account.period', 'End Period',
-            domain="[('fiscalyear', '=', fiscalyear), " \
-                    "('start_date', '>=', (start_period, 'start_date'))]",
+            domain=["('fiscalyear', '=', fiscalyear)",
+                "('start_date', '>=', (start_period, 'start_date'))"],
             depends=['start_period'])
     company = fields.Many2One('company.company', 'Company', required=True)
     posted = fields.Boolean('Posted Move', help='Show only posted move')
@@ -1537,12 +1612,12 @@ class OpenIncomeStatementInit(ModelView):
             required=True, on_change=['fiscalyear'],
             depends=['start_period', 'end_period'])
     start_period = fields.Many2One('account.period', 'Start Period',
-            domain="[('fiscalyear', '=', fiscalyear), " \
-                    "('start_date', '<=', (end_period, 'start_date'))]",
+            domain=["('fiscalyear', '=', fiscalyear)",
+                "('start_date', '<=', (end_period, 'start_date'))"],
             depends=['end_period'])
     end_period = fields.Many2One('account.period', 'End Period',
-            domain="[('fiscalyear', '=', fiscalyear), " \
-                    "('start_date', '>=', (start_period, 'start_date'))]",
+            domain=["('fiscalyear', '=', fiscalyear)",
+                "('start_date', '>=', (start_period, 'start_date'))"],
             depends=['start_period'])
     company = fields.Many2One('company.company', 'Company', required=True)
     posted = fields.Boolean('Posted Move', help='Show only posted move')
@@ -1679,11 +1754,11 @@ class CreateChartAccountPropertites(ModelView):
     company = fields.Many2One('company.company', 'Company')
     account_receivable = fields.Many2One('account.account',
             'Default Receivable Account',
-            domain="[('kind', '=', 'receivable'), ('company', '=', company)]",
+            domain=[('kind', '=', 'receivable'), "('company', '=', company)"],
             depends=['company'])
     account_payable = fields.Many2One('account.account',
             'Default Payable Account',
-            domain="[('kind', '=', 'payable'), ('company', '=', company)]",
+            domain=[('kind', '=', 'payable'), "('company', '=', company)"],
             depends=['company'])
 
 CreateChartAccountPropertites()
@@ -1743,6 +1818,9 @@ class CreateChartAccount(Wizard):
         tax_rule_line_template_obj = \
                 self.pool.get('account.tax.rule.line.template')
 
+        context=context.copy()
+        context['language'] = 'en_US'
+
         account_template = account_template_obj.browse(cursor, user,
                 datas['form']['account_template'], context=context)
 
@@ -1862,12 +1940,20 @@ class UpdateChartAccountInit(ModelView):
     'Update Chart Account from Template Init'
     _name = 'account.account.update_chart_account.init'
     _description = __doc__
-    account = fields.Many2One('account.account', 'Account',
+    account = fields.Many2One('account.account', 'Root Account',
             required=True, domain=[('parent', '=', False)])
 
 UpdateChartAccountInit()
 
 
+class UpdateChartAccountStart(ModelView):
+    'Update Chart Account from Template Start'
+    _name = 'account.account.update_chart_account.start'
+    _description = __doc__
+
+UpdateChartAccountStart()
+
+
 class UpdateChartAccount(Wizard):
     'Update Chart Account from Template'
     _name = 'account.account.update_chart_account'
@@ -1878,15 +1964,18 @@ class UpdateChartAccount(Wizard):
                 'object': 'account.account.update_chart_account.init',
                 'state': [
                     ('end', 'Cancel', 'tryton-cancel'),
-                    ('update_account', 'Ok', 'tryton-ok', True),
+                    ('start', 'Ok', 'tryton-ok', True),
                 ],
             },
         },
-        'update_account': {
+        'start': {
+            'actions': ['_action_update_account'],
             'result': {
-                'type': 'action',
-                'action': '_action_update_account',
-                'state': 'end',
+                'type': 'form',
+                'object': 'account.account.update_chart_account.start',
+                'state': [
+                    ('end', 'Ok', 'tryton-ok', True),
+                ],
             },
         },
     }
@@ -2298,7 +2387,7 @@ class AgedBalance(Report):
                         'JOIN account_account a ON (l.account = a.id) '
                     'WHERE l.party IS NOT NULL '\
                         'AND a.active ' \
-                        'AND a.kind IN ('+ ','.join('%s' for i in kind) + ") "\
+                        'AND a.kind IN ('+ ','.join(('%s',) * len(kind)) + ") "\
                         'AND l.reconciliation IS NULL ' \
                         'AND a.company = %s ' \
                         'AND '+ term_query+\
