@@ -29,13 +29,15 @@ class FiscalYear(ModelSQL, ModelView):
         ('close', 'Close'),
         ], 'State', readonly=True, required=True)
     post_move_sequence = fields.Many2One('ir.sequence', 'Post Move Sequence',
-            required=True, domain="[('code', '=', 'account.move')," \
-                    "['OR', ('company', '=', company)," \
-                    "('company', '=', False)]]",
+            required=True, domain=["('code', '=', 'account.move')",
+                ['OR',
+                    "('company', '=', company)",
+                    "('company', '=', False)"]],
                     context="{'code': 'account.move', 'company': company}",
             depends=['company'])
     company = fields.Many2One('company.company', 'Company', required=True,
-            domain="[('id', '=', context.get('company', False))]")
+            domain=["('id', 'company' in context and '=' or '!=', " \
+                    "context.get('company', 0))"])
 
     def __init__(self):
         super(FiscalYear, self).__init__()
@@ -90,7 +92,7 @@ class FiscalYear(ModelSQL, ModelView):
                         fiscalyear.end_date, fiscalyear.end_date,
                         fiscalyear.start_date, fiscalyear.end_date,
                         fiscalyear.company.id, fiscalyear.id))
-            if cursor.rowcount:
+            if cursor.fetchone():
                 return False
         return True
 
@@ -316,7 +318,7 @@ class CloseFiscalYearInit(ModelView):
     close_fiscalyear = fields.Many2One('account.fiscalyear',
             'Fiscal Year to close',
             required=True,
-            domain="[('state', '!=', 'close')]")
+            domain=["('state', '!=', 'close')"])
 
 CloseFiscalYearInit()
 
