@@ -30,7 +30,7 @@ class CodeTemplate(ModelSQL, ModelView):
     _name = 'account.tax.code.template'
     _description = __doc__
 
-    name = fields.Char('Name', required=True)
+    name = fields.Char('Name', required=True, translate=True)
     code = fields.Char('Code')
     parent = fields.Many2One('account.tax.code.template', 'Parent')
     childs = fields.One2Many('account.tax.code.template', 'parent', 'Children')
@@ -89,6 +89,7 @@ class CodeTemplate(ModelSQL, ModelView):
         :return: id of the tax code created
         '''
         tax_code_obj = self.pool.get('account.tax.code')
+        lang_obj = self.pool.get('ir.lang')
 
         if template2tax_code is None:
             template2tax_code = {}
@@ -103,6 +104,22 @@ class CodeTemplate(ModelSQL, ModelView):
             vals['parent'] = parent_id
 
             new_id = tax_code_obj.create(cursor, user, vals, context=context)
+
+            prev_lang = template._context.get('language') or 'en_US'
+            ctx = context.copy()
+            for lang in lang_obj.get_translatable_languages(cursor, user,
+                    context=context):
+                if lang == prev_lang:
+                    continue
+                ctx['language'] = lang
+                template.setLang(lang)
+                data = {}
+                for field in template._columns.keys():
+                    if template._columns[field].translate:
+                        data[field] = template[field]
+                if data:
+                    tax_code_obj.write(cursor, user, new_id, data, context=ctx)
+            template.setLang(prev_lang)
             template2tax_code[template.id] = new_id
         else:
             new_id = template2tax_code[template.id]
@@ -122,14 +139,15 @@ class Code(ModelSQL, ModelView):
     _name = 'account.tax.code'
     _description = __doc__
 
-    name = fields.Char('Name', size=None, required=True, select=1)
+    name = fields.Char('Name', size=None, required=True, select=1,
+                       translate=True)
     code = fields.Char('Code', size=None, select=1)
     active = fields.Boolean('Active', select=2)
     company = fields.Many2One('company.company', 'Company', required=True)
     parent = fields.Many2One('account.tax.code', 'Parent', select=1,
-            domain="[('company', '=', company)]", depends=['company'])
+            domain=["('company', '=', company)"], depends=['company'])
     childs = fields.One2Many('account.tax.code', 'parent', 'Children',
-            domain="[('company', '=', company)]", depends=['company'])
+            domain=["('company', '=', company)"], depends=['company'])
     currency_digits = fields.Function('get_currency_digits', type='integer',
             string='Currency Digits', on_change_with=['company'])
     sum = fields.Function('get_sum', digits="(16, currency_digits)",
@@ -189,12 +207,15 @@ class Code(ModelSQL, ModelView):
                 'WHERE c.id = tl.code ' \
                     'AND tl.move_line = l.id ' \
                     'AND c.id IN (' + \
-                        ','.join(['%s' for x in all_ids])+ ') ' \
+                        ','.join(('%s',) * len(all_ids))+ ') ' \
                     'AND ' + line_query + ' ' \
                     'AND c.active ' \
                 'GROUP BY c.id', all_ids)
         code_sum = {}
         for code_id, sum in cursor.fetchall():
+            # SQLite uses float for SUM
+            if not isinstance(sum, Decimal):
+                sum = Decimal(str(sum))
             code_sum[code_id] = sum
 
         for code in self.browse(cursor, user, ids, context=context):
@@ -257,6 +278,7 @@ class Code(ModelSQL, ModelView):
                 tax code. The dictionary is filled with new tax codes
         '''
         template_obj = self.pool.get('account.tax.code.template')
+        lang_obj = self.pool.get('ir.lang')
 
         if template2tax_code is None:
             template2tax_code = {}
@@ -269,6 +291,20 @@ class Code(ModelSQL, ModelView):
                     code.template, context=context, code=code)
             if vals:
                 self.write(cursor, user, code.id, vals, context=context)
+
+            prev_lang = code._context.get('language') or 'en_US'
+            ctx = context.copy()
+            for lang in lang_obj.get_translatable_languages(cursor, user,
+                    context=context):
+                if lang == prev_lang:
+                    continue
+                ctx['language'] = lang
+                code.setLang(lang)
+                data = template_obj._get_tax_code_value(cursor, user,
+                                code.template, context=ctx, code=code)
+                if data:
+                    self.write(cursor, user, code.id, data, context=ctx)
+            code.setLang(prev_lang)
             template2tax_code[code.template.id] = code.id
 
         for child in code.childs:
@@ -357,7 +393,7 @@ class TaxTemplate(ModelSQL, ModelView):
     description = fields.Char('Description', required=True, translate=True)
     group = fields.Many2One('account.tax.group', 'Group')
     sequence = fields.Integer('Sequence')
-    amount = fields.Numeric('Amount', digits=(16, 2))
+    amount = fields.Numeric('Amount', digits=(16, 8))
     percentage = fields.Numeric('Percentage', digits=(16, 8))
     type = fields.Selection([
         ('percentage', 'Percentage'),
@@ -404,16 +440,16 @@ class TaxTemplate(ModelSQL, ModelView):
         return False
 
     def default_invoice_base_sign(self, cursor, user, context=None):
-        return 1
+        return Decimal('1')
 
     def default_invoice_tax_sign(self, cursor, user, context=None):
-        return 1
+        return Decimal('1')
 
     def default_credit_note_base_sign(self, cursor, user, context=None):
-        return 1
+        return Decimal('1')
 
     def default_credit_note_tax_sign(self, cursor, user, context=None):
-        return 1
+        return Decimal('1')
 
     def _get_tax_value(self, cursor, user, template, context=None,
             tax=None):
@@ -466,6 +502,7 @@ class TaxTemplate(ModelSQL, ModelView):
         :return: id of the tax created
         '''
         tax_obj = self.pool.get('account.tax')
+        lang_obj = self.pool.get('ir.lang')
 
         if template2tax is None:
             template2tax = {}
@@ -509,6 +546,22 @@ class TaxTemplate(ModelSQL, ModelView):
                 vals['credit_note_tax_code'] = False
 
             new_id = tax_obj.create(cursor, user, vals, context=context)
+
+            prev_lang = template._context.get('language') or 'en_US'
+            ctx = context.copy()
+            for lang in lang_obj.get_translatable_languages(cursor, user,
+                    context=context):
+                if lang == prev_lang:
+                    continue
+                ctx['language'] = lang
+                template.setLang(lang)
+                data = {}
+                for field in template._columns.keys():
+                    if template._columns[field].translate:
+                        data[field] = template[field]
+                if data:
+                    tax_obj.write(cursor, user, new_id, data, context=ctx)
+            template.setLang(prev_lang)
             template2tax[template.id] = new_id
         else:
             new_id = template2tax[template.id]
@@ -545,10 +598,12 @@ class Tax(ModelSQL, ModelView):
     active = fields.Boolean('Active')
     sequence = fields.Integer('Sequence',
             help='Use to order the taxes')
-    amount = fields.Numeric('Amount', digits=(16, 2),
+    currency_digits = fields.Function('get_currency_digits', type='integer',
+            string='Currency Digits', on_change_with=['company'])
+    amount = fields.Numeric('Amount', digits="(16, currency_digits)",
             states={
                 'invisible': "type != 'fixed'",
-            }, help='In company\'s currency', depends=['type'])
+            }, help='In company\'s currency', depends=['type', 'currency_digits'])
     percentage = fields.Numeric('Percentage', digits=(16, 8),
             states={
                 'invisible': "type != 'percentage'",
@@ -561,17 +616,18 @@ class Tax(ModelSQL, ModelView):
     parent = fields.Many2One('account.tax', 'Parent', ondelete='CASCADE')
     childs = fields.One2Many('account.tax', 'parent', 'Children')
     company = fields.Many2One('company.company', 'Company', required=True,
-            domain="[('id', '=', context.get('company', 0))]")
+            domain=["('id', 'company' in context and '=' or '!=', " \
+                    "context.get('company', 0))"])
     invoice_account = fields.Many2One('account.account', 'Invoice Account',
-            domain="[('company', '=', company)]",
-            help='Leave empty to use the default invoice account',
+            domain=["('company', '=', company)",
+                    ('kind', '!=', 'view')],
             states={
                 'readonly': "type == 'none' or not company",
                 'required': "type != 'none' and company",
             }, depends=['company'])
     credit_note_account = fields.Many2One('account.account', 'Credit Note Account',
-            domain="[('company', '=', company)]",
-            help='Leave empty to use the default credit note account',
+            domain=["('company', '=', company)",
+                    ('kind', '!=', 'view')],
             states={
                 'readonly': "type == 'none' or not company",
                 'required': "type != 'none' and company",
@@ -640,16 +696,16 @@ class Tax(ModelSQL, ModelView):
         return False
 
     def default_invoice_base_sign(self, cursor, user, context=None):
-        return 1
+        return Decimal('1')
 
     def default_invoice_tax_sign(self, cursor, user, context=None):
-        return 1
+        return Decimal('1')
 
     def default_credit_note_base_sign(self, cursor, user, context=None):
-        return 1
+        return Decimal('1')
 
     def default_credit_note_tax_sign(self, cursor, user, context=None):
-        return 1
+        return Decimal('1')
 
     def default_company(self, cursor, user, context=None):
         company_obj = self.pool.get('company.company')
@@ -659,6 +715,21 @@ class Tax(ModelSQL, ModelView):
             return context['company']
         return False
 
+    def on_change_with_currency_digits(self, cursor, user, ids, vals,
+            context=None):
+        company_obj = self.pool.get('company.company')
+        if vals.get('company'):
+            company = company_obj.browse(cursor, user, vals['company'],
+                    context=context)
+            return company.currency.digits
+        return 2
+
+    def get_currency_digits(self, cursor, user, ids, name, arg, context=None):
+        res = {}
+        for tax in self.browse(cursor, user, ids, context=context):
+            res[tax.id] = tax.company.currency.digits
+        return res
+
     def _process_tax(self, cursor, user, tax, price_unit, context=None):
         if tax.type == 'percentage':
             amount = price_unit * tax.percentage / Decimal('100')
@@ -755,6 +826,7 @@ class Tax(ModelSQL, ModelView):
                 The dictionary is filled with new taxes
         '''
         template_obj = self.pool.get('account.tax.template')
+        lang_obj = self.pool.get('ir.lang')
 
         if template2tax is None:
             template2tax = {}
@@ -828,6 +900,20 @@ class Tax(ModelSQL, ModelView):
 
             if vals:
                 self.write(cursor, user, tax.id, vals, context=context)
+
+            prev_lang = tax._context.get('language') or 'en_US'
+            ctx = context.copy()
+            for lang in lang_obj.get_translatable_languages(cursor, user,
+                    context=context):
+                if lang == prev_lang:
+                    continue
+                ctx['language'] = lang
+                tax.setLang(lang)
+                data = template_obj._get_tax_value(cursor, user, tax.template,
+                        context=ctx, tax=tax)
+                if data:
+                    self.write(cursor, user, tax.id, data, context=ctx)
+            tax.setLang(prev_lang)
             template2tax[tax.template.id] = tax.id
 
         for child in tax.childs:
@@ -843,11 +929,29 @@ class Line(ModelSQL, ModelView):
     _description = __doc__
     _rec_name = 'amount'
 
-    amount = fields.Numeric('Amount', digits=(16, 2))
+    currency_digits = fields.Function('get_currency_digits', type='integer',
+            string='Currency Digits', on_change_with=['move_line'])
+    amount = fields.Numeric('Amount', digits="(16, currency_digits)",
+            depends=['currency_digits'])
     code = fields.Many2One('account.tax.code', 'Code', select=1, required=True)
     move_line = fields.Many2One('account.move.line', 'Move Line',
             required=True, select=1, ondelete='CASCADE')
 
+    def on_change_with_currency_digits(self, cursor, user, ids, vals,
+            context=None):
+        move_line_obj = self.pool.get('account.move.line')
+        if vals.get('move_line'):
+            move_line = move_line_obj.browse(cursor, user, vals['move_line'],
+                    context=context)
+            return move_line.currency_digits
+        return 2
+
+    def get_currency_digits(self, cursor, user, ids, name, arg, context=None):
+        res = {}
+        for line in self.browse(cursor, user, ids, context=context):
+            res[line.id] = line.move_line.currency_digits
+        return res
+
 Line()
 
 
@@ -855,7 +959,7 @@ class RuleTemplate(ModelSQL, ModelView):
     'Tax Rule Template'
     _name = 'account.tax.rule.template'
     _description = __doc__
-    name = fields.Char('Name', required=True)
+    name = fields.Char('Name', required=True, translate=True)
     lines = fields.One2Many('account.tax.rule.line.template', 'rule', 'Lines')
     account = fields.Many2One('account.account.template', 'Account Template',
             domain=[('parent', '=', False)], required=True)
@@ -897,6 +1001,7 @@ class RuleTemplate(ModelSQL, ModelView):
         :return: id of the tax rule created
         '''
         rule_obj = self.pool.get('account.tax.rule')
+        lang_obj = self.pool.get('ir.lang')
 
         if template2rule is None:
             template2rule = {}
@@ -909,6 +1014,22 @@ class RuleTemplate(ModelSQL, ModelView):
                     context=context)
             vals['company'] = company_id
             new_id = rule_obj.create(cursor, user, vals, context=context)
+
+            prev_lang = template._context.get('language') or 'en_US'
+            ctx = context.copy()
+            for lang in lang_obj.get_translatable_languages(cursor, user,
+                    context=context):
+                if lang == prev_lang:
+                    continue
+                ctx['language'] = lang
+                template.setLang(lang)
+                data = {}
+                for field in template._columns.keys():
+                    if template._columns[field].translate:
+                        data[field] = template[field]
+                if data:
+                    rule_obj.write(cursor, user, new_id, data, context=ctx)
+            template.setLang(prev_lang)
             template2rule[template.id] = new_id
         else:
             new_id = template2rule[template.id]
@@ -921,9 +1042,10 @@ class Rule(ModelSQL, ModelView):
     'Tax Rule'
     _name = 'account.tax.rule'
     _description = __doc__
-    name = fields.Char('Name', required=True)
+    name = fields.Char('Name', required=True, translate=True)
     company = fields.Many2One('company.company', 'Company', required=True,
-            select=1, domain="[('id', '=', context.get('company', 0))]")
+            select=1, domain=["('id', 'company' in context and '=' or '!=', " \
+                    "context.get('company', 0))"])
     lines = fields.One2Many('account.tax.rule.line', 'rule', 'Lines')
     template = fields.Many2One('account.tax.rule.template', 'Template')
 
@@ -943,14 +1065,15 @@ class Rule(ModelSQL, ModelView):
         tax_obj = self.pool.get('account.tax')
         rule_line_obj = self.pool.get('account.tax.rule.line')
 
-        if isinstance(rule, (int, long)):
+        if isinstance(rule, (int, long)) and rule:
             rule = self.browse(cursor, user, rule, context=context)
 
-        if isinstance(tax, (int, long)):
+        if isinstance(tax, (int, long)) and tax:
             tax = tax_obj.browse(cursor, user, tax, context=context)
 
         pattern = pattern.copy()
         pattern['group'] = tax and tax.group.id or False
+        pattern['origin_tax'] = tax and tax.id or False
 
         for line in rule.lines:
             if rule_line_obj.match(cursor, user, line, pattern,
@@ -972,6 +1095,7 @@ class Rule(ModelSQL, ModelView):
                 tax rule. The dictionary is filled with new tax rules
         '''
         template_obj = self.pool.get('account.tax.rule.template')
+        lang_obj = self.pool.get('ir.lang')
 
         if template2rule is None:
             template2rule = {}
@@ -984,6 +1108,20 @@ class Rule(ModelSQL, ModelView):
                     rule.template, context=context, rule=rule)
             if vals:
                 self.write(cursor, user, rule.id, vals, context=context)
+
+            prev_lang = rule._context.get('language') or 'en_US'
+            ctx = context.copy()
+            for lang in lang_obj.get_translatable_languages(cursor, user,
+                    context=context):
+                if lang == prev_lang:
+                    continue
+                ctx['language'] = lang
+                rule.setLang(lang)
+                data = template_obj._get_tax_rule_value(cursor, user,
+                        rule.template, context=ctx, rule=rule)
+                if data:
+                    self.write(cursor, user, rule.id, data, context=ctx)
+            rule.setLang(prev_lang)
             template2rule[rule.template.id] = rule.id
 
 Rule()
@@ -996,9 +1134,14 @@ class RuleLineTemplate(ModelSQL, ModelView):
     rule = fields.Many2One('account.tax.rule.template', 'Rule', required=True,
             ondelete='CASCADE')
     group = fields.Many2One('account.tax.group', 'Tax Group')
+    origin_tax = fields.Many2One('account.tax.template', 'Original Tax',
+            domain=["('account', '=', _parent_rule.account)",
+                "('group', '=', group)"],
+            help='If the original tax template is filled, the rule will be ' \
+                    'applied only for this tax template.')
     tax = fields.Many2One('account.tax.template', 'Substitution Tax',
-            domain="[('account', '=', _parent_rule.account), " \
-                    "('group', '=', group)]")
+            domain=["('account', '=', _parent_rule.account)",
+                "('group', '=', group)"])
     sequence = fields.Integer('Sequence')
 
     def __init__(self):
@@ -1021,6 +1164,8 @@ class RuleLineTemplate(ModelSQL, ModelView):
         res = {}
         if not rule_line or rule_line.group.id != template.group.id:
             res['group'] = template.group.id
+        if not rule_line or rule_line.origin_tax.id != template.origin_tax.id:
+            res['origin_tax'] = template.origin_tax.id
         if not rule_line or rule_line.sequence != template.sequence:
             res['sequence'] = template.sequence
         if not rule_line or rule_line.template.id != template.id:
@@ -1061,6 +1206,10 @@ class RuleLineTemplate(ModelSQL, ModelView):
             vals = self._get_tax_rule_line_value(cursor, user, template,
                     context=context)
             vals['rule'] = template2rule[template.rule.id]
+            if template.origin_tax:
+                vals['origin_tax'] = template2tax[template.origin_tax.id]
+            else:
+                vals['origin_tax'] = False
             if template.tax:
                 vals['tax'] = template2tax[template.tax.id]
             else:
@@ -1082,9 +1231,14 @@ class RuleLine(ModelSQL, ModelView):
     rule = fields.Many2One('account.tax.rule', 'Rule', required=True,
             select=1, ondelete='CASCADE')
     group = fields.Many2One('account.tax.group', 'Tax Group')
+    origin_tax = fields.Many2One('account.tax', 'Original Tax',
+            domain=["('company', '=', _parent_rule.company)",
+                "('group', '=', group)"],
+            help='If the original tax is filled, the rule will be applied ' \
+                    'only for this tax.')
     tax = fields.Many2One('account.tax', 'Substitution Tax',
-            domain="[('company', '=', _parent_rule.company), " \
-                    "('group', '=', group)]")
+            domain=["('company', '=', _parent_rule.company)",
+                "('group', '=', group)"])
     sequence = fields.Integer('Sequence')
     template = fields.Many2One('account.tax.rule.line.template', 'Template')
 
@@ -1168,6 +1322,12 @@ class RuleLine(ModelSQL, ModelView):
                     rule_line.template, context=context, rule_line=rule_line)
             if rule_line.rule.id != template2rule[rule_line.template.rule.id]:
                 vals['rule'] = template2rule[rule_line.template.rule.id]
+            if rule_line.origin_tax:
+                if rule_line.template.origin_tax:
+                    if rule_line.origin_tax.id != \
+                            template2tax[rule_line.template.origin_tax.id]:
+                        vals['origin_tax'] = template2tax[
+                            rule_line.template.origin_tax.id]
             if rule_line.tax:
                 if rule_line.template.tax:
                     if rule_line.tax.id != \
@@ -1255,7 +1415,7 @@ class AccountTemplate(ModelSQL, ModelView):
     _name = 'account.account.template'
     taxes = fields.Many2Many('account.account.template-account.tax.template',
             'account', 'tax', 'Default Taxes',
-            domain="[('parent', '=', False)]")
+            domain=["('parent', '=', False)"])
 
 AccountTemplate()
 
@@ -1277,7 +1437,7 @@ class Account(ModelSQL, ModelView):
     _name = 'account.account'
     taxes = fields.Many2Many('account.account-account.tax',
             'account', 'tax', 'Default Taxes',
-            domain="[('company', '=', company), ('parent', '=', False)]",
+            domain=["('company', '=', company)", ('parent', '=', False)],
             help='Default tax for manual encoding of move lines \n' \
                     'for journal types: "expense" and "revenue"',
             depends=['company'])
